/// `typos_cli`'s API is unstable.  Open an issue for starting a discussion on getting a subset
/// stabilized.

#[doc(hidden)]
pub mod config;
#[doc(hidden)]
pub mod dict;
#[doc(hidden)]
pub mod file;
#[doc(hidden)]
pub mod policy;
#[doc(hidden)]
pub mod report;

mod default_types;
mod file_type;
